/*
 * pdsnivector.h
 * 
 * Copyright 2011 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdsnivector.h
 *  \author Fernando Pujaico Rivera
 *  \date 05-01-2012
 *  \brief Funciones que trabajan con vectores de neuronas de entrada PdsNInput.
 *
 *  <br>Estas funciones trabajan con vectores de neuronas de la forma.<br>
 *  <center>
 *  \image html nivector.png "vetor de Nel neuronas"
 *  </center>
 */

#ifndef __PDSNIVECTOR_H__
#define __PDSNIVECTOR_H__

#ifdef __cplusplus
extern "C" {
#endif 

#include <stdio.h>
#include <stdlib.h>
#include <pds/pdssnglobal.h>
#include <pds/pdsninput.h>


#ifndef TRUE
	#define TRUE 1
#endif

#ifndef FALSE
	#define FALSE 0
#endif


/** \defgroup PdsNIVectorGroup Módulo PdsNIVector.
 *  \brief Funciones que trabajan con vectores de neuronas de entrada PdsNInput.
 *
 *  <br>Estas funciones trabajan con vectores de neuronas de la forma.<br>
 *  <center>
 *  \image html nivector.png "vetor de Nel neuronas"
 *  </center>
 * @{
 */


/*! \struct PdsNIVector
 *  \brief La estructura tipo  PdsNIVector .
 *  Esta estructura genera un arreglo de neuronas.
 *  Para usar incluir pds/pdsnn.h.
 *  \brief Funciones que trabajan con vectores de neuronas de entrada PdsNInput.
 *  
 *  <br>Estas funciones trabajan con vectores de neuronas de la forma.<br>
 *  <center>
 *  \image html nivector.png "vetor de Nel neuronas"
 *  </center>
 *  \ingroup PdsNIVectorGroup
 *  \author Fernando Pujaico Rivera
 */
typedef struct 
{
	/*! Arreglo de neuronas de entrada. */
	PdsNInput **V;
	/*! Número de neuronas de entrada del vector. */
	PdsSnNatural Nel;
}PdsNIVector;


/** @name pds_nivector_<new>
 *  Creando un vector de neuronas de entrada.
 *
 *  <center>
 *  \image html nivector.png "vetor de Nel neuronas"
 *  </center>
 *  <br>
 * @{
 */

/** \fn PdsNIVector *pds_nivector_new(PdsSnNatural Nel)
 *  \brief Crea un vector de neuronas de entrada de tipo PdsNIVector e inicia 
 *  con cero todos los valores.
 *  \param[in] Nel Es el número de elementos del vector.
 *  \return Un puntero al vector de tipo PdsNIVector.
 *  \ingroup PdsNIVectorGroup
 */
PdsNIVector *pds_nivector_new(PdsSnNatural Nel);


//@}

/** @name pds_nivector_<get_set_methods>
 *  Leer y escribir datos datos.
 *
 * @{
 */


/** \fn int pds_nivector_set_input_from_array(PdsNIVector *NIVector,const PdsSnReal *X,int Nel)
 *  \brief Escribe el valor de la entrada de las Neuronas.
 *  \param[in,out] NIVector La neurona en consulta.
 *  \param[in] X Arreglo donde se lee los valores de la entrada.
 *  \param[in] Nel Número de elementos em X.
 *  Solo se copian en NIVector los elementos que caben.
 *  \return TRUE si todo fue bien o FALSE si no (ej: NIVector==NULL, X==NULL).
 *  \ingroup PdsNIVectorGroup
 */
int pds_nivector_set_input_from_array(PdsNIVector *NIVector,const PdsSnReal *X,int Nel);

//@}

/** @name pds_nivector_<printf>
 *  Otros métodos.
 *
 * @{
 */

/** \fn int pds_nivector_printf(const PdsNIVector *NIVector)
 *  \brief Imprime el valor de la entrada de las Neuronas.
 *  \param[in] NIVector La neurona en consulta.
 *  \return TRUE si todo fue bien o FALSE si no (ej: NIVector==NULL)
 *  \ingroup PdsNIVectorGroup
 */
int pds_nivector_printf(const PdsNIVector *NIVector);

//@}

/** @name pds_nivector_<methods>
 *  Otros métodos.
 *
 * @{
 */


/** \fn int pds_nivector_iterate(PdsNIVector *NIVector)
 *  \brief Escribe el valor cero en todas las segundas salidas (error) de cada elemento 
 *  del vector. \f$e_k=0.0\f$.
 *  \param[in,out] NIVector La neurona en consulta.
 *  \return TRUE si todo fue bien o FALSE si no (ej: NIVector==NULL). 
 *  \ingroup PdsNIVectorGroup
 */
int pds_nivector_iterate(PdsNIVector *NIVector);


//@}

/** @name pds_nivector_<free>
 *  Liberar memoria.
 *
 * @{
 */


/** \fn void pds_nivector_free(PdsNIVector *NIVector)
 *  \brief Libera un vector de neuronas de tipo puntero PdsNIVector.
 *  \param[in,out] NIVector el vector de neuronas a liberar.
 *  \return No retorna valor.
 *  \ingroup PdsNIVectorGroup
 */
void pds_nivector_free(PdsNIVector *NIVector);


/** \fn void pds_nivector_destroy(PdsNIVector **NIVector)
 *  \brief Libera un vector de neuronas de tipo puntero PdsNIVector, y limpia el 
 *   puntero con NULL.
 *  \param[in,out] NIVector El vector de neuronas a liberar y limpiar.
 *  \return No retorna valor.
 *  \ingroup PdsNIVectorGroup
 */
void pds_nivector_destroy(PdsNIVector **NIVector);


//@}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif 

#endif


